@@ -55,6 +55,8 @@ class ActionRules:
         The target values that are not in the before part.
     decisions: Decisions()
         Decisions object.
+    is_strict_flexible : bool
+        If true flexible attributes must be always actionable, if false they can also behave as stable attributes
 
     Methods
     -------
@@ -80,6 +82,7 @@ class ActionRules:
                  min_flexible_antecedents: int = 1,
                  max_stable_antecedents: int = 1,
                  max_flexible_antecedents: int = 1,
+                 is_strict_flexible: bool = True
                  ):
         """
         Parameters
@@ -108,7 +111,8 @@ class ActionRules:
             Maximal number of stable pairs.
         max_flexible_antecedents : int
             Maximal number of flexible pairs.
-
+        is_strict_flexible : bool
+            If true flexible attributes must be always actionable, if false they can also behave as stable attributes
         """
         self.stable_tables = stable_tables
         self.flexible_tables = flexible_tables
@@ -130,6 +134,7 @@ class ActionRules:
         self.desired_target_classes = self.desired_state.get_destination_classes()
         self.not_default_target_classes = self.desired_state.get_not_in_default_classes()
         self.decisions = decisions
+        self.is_strict_flexible = is_strict_flexible
 
     def _is_action_couple(self,
                           before: Union[str, int, float],
@@ -152,8 +157,8 @@ class ActionRules:
         tuple
             Returns (bool is_action_pair, (before, after) action_pair, bool break_rule).
         """
-        before = str(before)
-        after = str(after)
+        before = str(before).lower()
+        after = str(after).lower()
         if attribute_type == "stable":
             if before == "nan" and after == "nan":
                 return False, None, False
@@ -169,6 +174,9 @@ class ActionRules:
                 return False, None, False
             elif before != after and before != "nan" and after != "nan":
                 return True, (before, after), False
+            elif not self.is_strict_flexible:
+                if before == after and before != "nan":
+                    return False, (before,), False
             elif self.is_nan:
                 if before != after and before == "nan":
                     return True, (str(None), after), False
@@ -216,8 +224,7 @@ class ActionRules:
             elif is_action_couple:
                 count_antecedent += 1
                 action_rule_part.append([column, action_couple])
-            else:
-                if action_couple is not None:
+            elif action_couple is not None:
                     action_rule_part.append([column, action_couple])
         return True, action_rule_part, count_antecedent
 
@@ -226,7 +233,8 @@ class ActionRules:
                          action_rule_flexible: list,
                          action_rule_decision: list,
                          action_rule_supp: list,
-                         action_rule_conf: list):
+                         action_rule_conf: list,
+                         uplift: float):
         """This method joins the parts of an action rule and adds the action rule to a list.
 
         Parameters
@@ -241,9 +249,10 @@ class ActionRules:
             List of supports.
         action_rule_conf : list
             List of confidences.
+        uplift: float
+            Uplift
         """
         action_rule = [action_rule_stable, action_rule_flexible, action_rule_decision]
-        uplift = self._get_uplift(action_rule_supp[0], action_rule_conf[0], action_rule_conf[1])
         self.action_rules.append([action_rule, action_rule_supp, action_rule_conf, uplift])
 
     def _split_to_before_after_consequent(self, decision_column: pd.DataFrame) -> tuple:
@@ -316,11 +325,17 @@ class ActionRules:
                         if not self.is_nan:
                             support = min(supp[rule_before_index], supp[rule_after_index])
                             confidence = conf[rule_before_index] * conf[rule_after_index]
+                            uplift = self._get_uplift(
+                                supp[rule_before_index],
+                                conf[rule_before_index],
+                                conf[rule_after_index]
+                            )
                         else:
                             total = len(self.decisions.transactions)
                             if total == 0:
                                 support = None
                                 confidence = None
+                                uplift = None
                             else:
                                 (left_support_before, support_before) = self._get_frequency_from_mask(action_rule_stable,
                                                                                                       action_rule_flexible,
@@ -333,7 +348,16 @@ class ActionRules:
                                                                                                     1
                                                                                                     )
                                 support = support_before / total
-                                confidence = (support_before / left_support_before) * (support_after / left_support_after)
+                                if left_support_before != 0 and left_support_after != 0:
+                                    confidence = (support_before / left_support_before) * (support_after / left_support_after)
+                                    uplift = self._get_uplift(
+                                        support_before,
+                                        (support_before / left_support_before),
+                                        (support_after / left_support_after)
+                                    )
+                                else:
+                                    confidence = 0
+                                    uplift = 0
                         action_rule_supp = [supp[rule_before_index],
                                             supp[rule_after_index],
                                             support
@@ -346,7 +370,8 @@ class ActionRules:
                                               action_rule_flexible,
                                               action_rule_decision,
                                               action_rule_supp,
-                                              action_rule_conf)
+                                              action_rule_conf,
+                                              uplift)
                         self.classification_before.append(rule_before_index)
                         self.classification_after.append(rule_after_index)
 
@@ -367,8 +392,12 @@ class ActionRules:
             # Flexible part
             flexible_part = action_rule[1]
             for flexible_couple in flexible_part:
-                text += "attribute '" + str(flexible_couple[0]) + "' value '" + str(flexible_couple[1][0]) + \
-                        "' is changed to '" + str(flexible_couple[1][1]) + "', "
+                if len(flexible_couple[1]) == 2:
+                    text += "attribute '" + str(flexible_couple[0]) + "' value '" + str(flexible_couple[1][0]) + \
+                            "' is changed to '" + str(flexible_couple[1][1]) + "', "
+                else:
+                    text += "attribute '" + str(flexible_couple[0]) + "' value '" + str(flexible_couple[1][0]) + \
+                            "' remains the same, "
             # Decision
             decision = action_rule[2]
             text += "then '" + str(decision[0]) + "' value '" + str(decision[1][0]) + "' is changed to '" + \
@@ -395,8 +424,11 @@ class ActionRules:
             flexible_part = action_rule[1]
             text = text[:-3]
             for flexible_couple in flexible_part:
-                text += " ∧ (" + str(flexible_couple[0]) + ": " + str(flexible_couple[1][0]) + \
-                        " → " + str(flexible_couple[1][1]) + ") "
+                if len(flexible_couple[1]) == 2:
+                    text += " ∧ (" + str(flexible_couple[0]) + ": " + str(flexible_couple[1][0]) + \
+                            " → " + str(flexible_couple[1][1]) + ") "
+                else:
+                    text += " ∧  (" + str(flexible_couple[0]) + ": " + str(flexible_couple[1][0]) + ") "
             # Decision
             decision = action_rule[2]
             text += "] ⇒ [" + str(decision[0]) + ": " + str(decision[1][0]) + " → " + \
@@ -424,7 +456,10 @@ class ActionRules:
         float
             An uplift value.
         """
-        return ((supp_before / conf_before) * conf_after) - ((supp_before / conf_before) - supp_before)
+        if conf_before != 0:
+            return ((supp_before / conf_before) * conf_after) - ((supp_before / conf_before) - supp_before)
+        else:
+            return 0
 
     def _get_frequency_from_mask(self, action_rule_stable: list, action_rule_flexible: list, action_rule_decision: list, part: int) -> tuple:
         """Get frequency from source data.
