@@ -164,6 +164,7 @@ class ActionRulesDiscovery:
             min_flexible_attributes: int = 1,
             max_stable_attributes: int = 5,
             max_flexible_attributes: int = 5,
+            is_strict_flexible: bool = True
             ):
         """Train the model from transaction data.
 
@@ -186,6 +187,8 @@ class ActionRulesDiscovery:
         - min_flexible_attributes
         - max_stable_attributes
         - max_flexible_attributes
+        The way how the flexible attribute behave
+        - is_strict_flexible
 
         Parameters
         ----------
@@ -227,11 +230,12 @@ class ActionRulesDiscovery:
         max_flexible_attributes : int = 5
             Maximal number of flexible pairs.
             DEFAULT: 5
+        is_strict_flexible : bool = True
+            If true flexible attributes must be always actionable, if false they can also behave as stable attributes
+            DEFAULT: True
         """
         if (self.action_rules):
             raise Exception("Fit was already called")
-        self.stable_attributes = stable_attributes
-        self.flexible_attributes = flexible_attributes
         self.consequent = consequent
         if bool(desired_classes) != bool(desired_changes):
             self.desired_state = DesiredState(desired_classes=desired_classes, desired_changes=desired_changes)
@@ -242,8 +246,12 @@ class ActionRulesDiscovery:
         self.decisions.prepare_data_fim(attributes, consequent)
         self.decisions.fit_fim_apriori(conf=conf, support=supp)
         self.decisions.generate_decision_table()
-        stable = self.decisions.decision_table[stable_attributes]
-        flex = self.decisions.decision_table[flexible_attributes]
+        # Not all columns are in the generated classification rules
+        self.stable_attributes = list(set(stable_attributes).intersection(set(self.decisions.decision_table.columns)))
+        self.flexible_attributes = list(set(flexible_attributes).intersection(set(self.decisions.decision_table.columns)))
+        # Data
+        stable = self.decisions.decision_table[self.stable_attributes]
+        flex = self.decisions.decision_table[self.flexible_attributes]
         target = self.decisions.decision_table[[consequent]]
         supp = self.decisions.support
         conf = self.decisions.confidence
@@ -262,7 +270,8 @@ class ActionRulesDiscovery:
             min_stable_attributes,
             min_flexible_attributes,
             max_stable_attributes,
-            max_flexible_attributes
+            max_flexible_attributes,
+            is_strict_flexible
         )
         self.action_rules.fit()
 
@@ -280,6 +289,7 @@ class ActionRulesDiscovery:
                                  min_flexible_attributes: int = 1,
                                  max_stable_attributes: int = 5,
                                  max_flexible_attributes: int = 5,
+                                 is_strict_flexible: bool = True
                                  ):
         """Train the model from classification rules.
 
@@ -302,6 +312,8 @@ class ActionRulesDiscovery:
         - min_flexible_attributes
         - max_stable_attributes
         - max_flexible_attributes
+        The way how the flexible attribute behave
+        - is_strict_flexible
 
         Parameters
         ----------
@@ -342,6 +354,9 @@ class ActionRulesDiscovery:
         max_flexible_attributes: int = 5
             Maximal number of flexible pairs.
             DEFAULT: 5
+        is_strict_flexible : bool = True
+            If true flexible attributes must be always actionable, if false they can also behave as stable attributes
+            DEFAULT: True
         """
         if (self.action_rules):
             raise Exception("Fit was already called")
@@ -378,7 +393,8 @@ class ActionRulesDiscovery:
             min_stable_attributes,
             min_flexible_attributes,
             max_stable_attributes,
-            max_flexible_attributes
+            max_flexible_attributes,
+            is_strict_flexible
         )
         self.action_rules.fit()
 
@@ -476,7 +492,7 @@ class ActionRulesDiscovery:
         """
         new_data = source_table.applymap(str).copy()
         for key, value in decision.items():
-            if str(value) != "nan":
+            if str(value).lower() != "nan":
                 mask = new_data[key] == value
                 new_data = new_data[mask]
         return new_data
@@ -505,7 +521,7 @@ class ActionRulesDiscovery:
             predicted_table = self._reduce_table_source(decision_before, source_table)
             if len(predicted_table.index) > 0:
                 for key, value in decision_after.items():
-                    if str(value) != "nan" and key in self.flexible_attributes:
+                    if str(value).lower() != "nan" and key in self.flexible_attributes:
                         column = key + self.RECOMMENDED
                         predicted_table[column] = [value] * len(predicted_table.index)
                         predicted_table[self.ACTION_RULE] = [i] * len(predicted_table.index)
