@@ -127,10 +127,17 @@ class Reduction:
         for unique_value in unique_values:
             if self.is_nan:
                 mask = np.logical_or(stable_columns.iloc[:, split_position] == unique_value,
-                                     stable_columns.iloc[:, split_position] == 'nan'
+                                     np.logical_or(stable_columns.iloc[:, split_position].isnull(),
+                                                   stable_columns.iloc[:, split_position] == 'nan'
+                                                   )
                                      )
             else:
-                mask = stable_columns.iloc[:, split_position] == unique_value
+                if str(unique_value).lower() == "nan":
+                    mask = np.logical_or(stable_columns.iloc[:, split_position].isnull(),
+                                         stable_columns.iloc[:, split_position] == 'nan'
+                                         )
+                else:
+                    mask = stable_columns.iloc[:, split_position] == unique_value
             new_stable_table = stable_columns[mask]
             new_flexible_table = flexible_columns[mask]
             new_decision_table = decision_column[mask]
